<?php
namespace Ak\Orm\Service;

use Interop\Container\ContainerInterface;
use Laminas\ServiceManager\Exception\ServiceNotCreatedException;
use Laminas\ServiceManager\Factory\FactoryInterface;
use Psr\Cache\CacheItemPoolInterface;
use Symfony\Component\Cache\Adapter\ArrayAdapter;
use Symfony\Component\Cache\Adapter\FilesystemAdapter;

/**
 * Stvara novu instancu keša meta-podataka.
 *
 * @package Ak\Orm\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class CacheFactory implements FactoryInterface
{
    /**
     * Stvara keš.
     *
     * @param ContainerInterface $container
     * @param string $requestedName
     * @param null|array $options
     * @return CacheItemPoolInterface
     */
    public function __invoke(ContainerInterface $container, $requestedName, array $options = null): CacheItemPoolInterface
    {
        $config = $container->get('config');

        if ($config['development'] ?? false) {
            return new ArrayAdapter();
        }

        $config = $config['orm']['cache'] ?? [];

        switch ($config['name'] ?? null) {
            case 'filesystem':
                $options = $config['options'] ?? ['directory' => sys_get_temp_dir()];

                if (!isset($options['directory']) || empty($options['directory'])) {
                    throw new ServiceNotCreatedException('Direktorij do keša meta-podataka nije postavljen.');
                }

                return new FilesystemAdapter(
                    $options['namespace'] ?? '',
                    $options['defaultLifetime'] ?? 0,
                    $options['directory'] ?? [sys_get_temp_dir()]
                );
            default:
                return new ArrayAdapter();
        }
    }
}
