<?php
namespace AkTest\Orm\Entity;

use AkCustomer\Entity\Customer;
use AkEmployee\Entity\Developer;
use AkTest\Orm\AbstractTestCase;
use Doctrine\DBAL\Driver\Exception as DBALDriverException;
use Doctrine\DBAL\Exception as DBALException;
use Doctrine\ORM\OptimisticLockException;
use Doctrine\ORM\ORMException;

final class SIngleTableInheritanceTest extends AbstractTestCase
{

    /**
     * @throws OptimisticLockException
     * @throws ORMException
     * @throws DBALException
     * @throws DBALDriverException
     */
    public function testCanSaveParentEntities()
    {
        $em = $this->getEntityManager();
        $person = new Person();

        $em->persist($person);
        $em->flush();

        $conn = $em->getConnection();
        $query = $conn->executeQuery('SELECT * FROM person');
        $rows = $query->fetchAllAssociative();

        $this->assertCount(1, $rows);
        $this->assertEquals(1, $rows[0]['id']);
        $this->assertEquals('person', $rows[0]['type']);
    }

    /**
     * @throws ORMException
     * @throws OptimisticLockException
     * @throws DBALDriverException
     * @throws DBALException
     */
    public function testCanSaveChildEntities()
    {
        $em = $this->getEntityManager();
        $person = new Employee();

        $em->persist($person);
        $em->flush();

        $conn = $em->getConnection();
        $query = $conn->executeQuery('SELECT * FROM person');
        $rows = $query->fetchAllAssociative();

        $this->assertCount(1, $rows);
        $this->assertEquals(1, $rows[0]['id']);
        $this->assertEquals('person_employee', $rows[0]['type']);
    }

    /**
     * @throws ORMException
     * @throws OptimisticLockException
     * @throws DBALDriverException
     * @throws DBALException
     */
    public function testCanWorkWithInheritanceAcrossNamespaces()
    {
        $em = $this->getEntityManager();
        $customer = new Customer();

        $em->persist($customer);
        $em->flush();

        $conn = $this->getConnection();
        $row = $conn->executeQuery('SELECT * FROM person WHERE id = 1')->fetchAssociative();

        $this->assertEquals('customer', $row['type']);

        $entity = $em->find(Person::class, 1);

        $this->assertInstanceOf(Customer::class, $entity);
    }

    /**
     * @throws ORMException
     * @throws OptimisticLockException
     * @throws DBALDriverException
     * @throws DBALException
     */
    public function testCanWorkWithInheritanceAcrossMultipleNamespaces()
    {
        $em = $this->getEntityManager();
        $developer = new Developer();

        $em->persist($developer);
        $em->flush();

        $conn = $this->getConnection();
        $row = $conn->executeQuery('SELECT * FROM person WHERE id = 1')->fetchAssociative();

        $this->assertEquals('developer', $row['type']);

        $entity = $em->find(Person::class, 1);

        $this->assertInstanceOf(Developer::class, $entity);
    }
}
