<?php
namespace AkTest\Orm\Service;

use Ak\Orm\ConfigProvider;
use Ak\Orm\Service\CacheFactory;
use Ak\Test\AbstractTestCase;
use AkTest\Orm\TestConfigProvider;
use Laminas\ServiceManager\Exception\ServiceNotCreatedException;
use Psr\Cache\CacheItemPoolInterface;
use Symfony\Component\Cache\Adapter\ArrayAdapter;
use Symfony\Component\Cache\Adapter\FilesystemAdapter;

/**
 * Class MetadataCacheFactoryTest
 *
 * @package AkTest\Orm\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class CacheTest extends AbstractTestCase
{
    protected array $providers = [
        ConfigProvider::class,
        TestConfigProvider::class,
    ];

    public function testIsRegistered()
    {
        $container = $this->getContainer();

        $this->assertContainerHasService($container, 'orm.cache', CacheItemPoolInterface::class);
    }

    public function testWillCreateArrayCacheWhenInDevelopmentMode()
    {
        $container = $this->getContainer([
            'development' => true,
        ]);
        $factory = new CacheFactory();

        $cache = $factory($container, '');

        $this->assertInstanceOf(ArrayAdapter::class, $cache);
    }

    public function testWillThrowExceptionIfFilesystemCacheDirectoryNotSet()
    {
        $this->expectException(ServiceNotCreatedException::class);

        $container = $this->getContainer([
            'development' => false,

            'orm' => [
                'cache' => [
                    'options' => [
                        'directory' => null,
                    ],
                ],
            ],
        ]);
        $factory = new CacheFactory();

        $factory($container, '');
    }

    public function testCanCreateCache()
    {
        $container = $this->getContainer([
            'development' => false,
        ]);
        $factory = new CacheFactory();

        $cache = $factory($container, '');

        $this->assertInstanceOf(FilesystemAdapter::class, $cache);
    }

    public function testCanCreateCacheWhenNoneConfigured()
    {
        $container = $this->getContainer([
            'orm' => [
                'metadata_cache' => null,
            ],
        ]);
        $factory = new CacheFactory();

        $cache = $factory($container, '');

        $this->assertInstanceOf(CacheItemPoolInterface::class, $cache);
    }
}
