<?php
namespace AkTest\Orm\Service;

use AkTest\Orm\AbstractTestCase;
use AkTest\Orm\Entity\ConcreteEntity;
use Doctrine\Persistence\Mapping\Driver\MappingDriverChain;
use Doctrine\ORM\Mapping\ClassMetadata;
use Doctrine\Persistence\Mapping\MappingException;
use Gedmo\Tree\Entity\MappedSuperclass\AbstractClosure;

/**
 * Class MappingDriverTest
 *
 * @package Ak\Orm\Service
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class MappingDriverTest extends AbstractTestCase
{
    public function testIsRegistered()
    {
        $container = $this->getContainer();

        $this->assertContainerHasService($container, 'orm.mapping_driver', MappingDriverChain::class);
    }

    /**
     * @throws MappingException
     */
    public function testGedmoExtensionsAreRegistered()
    {
        $container = $this->getContainer();
        /** @var MappingDriverChain $chain */
        $chain = $container->get('orm.mapping_driver');

        $chain->loadMetadataForClass(AbstractClosure::class, new ClassMetadata(AbstractClosure::class));
        $this->addToAssertionCount(1);
    }

    /**
     * @throws MappingException
     */
    public function testDefaultDriverIsRegistered()
    {
        $container = $this->getContainer();
        /** @var MappingDriverChain $chain */
        $chain = $container->get('orm.mapping_driver');

        $chain->loadMetadataForClass(ConcreteEntity::class, new ClassMetadata(ConcreteEntity::class));
        $this->addToAssertionCount(1);
    }
}
