<?php
namespace AkTest\Orm;

use Ak\Orm\ConfigProvider;
use Ak\Orm\Form\EntityForm;
use AkTest\Orm\Entity\ConcreteEntity;
use AkTest\Orm\Entity\TimestampableEntity;
use Laminas\ConfigAggregator\ArrayProvider;
use Laminas\ConfigAggregator\ConfigAggregator;
use Laminas\EventManager\EventManager;

final class TestConfigProvider extends ConfigAggregator
{
    public function __construct()
    {
        parent::__construct([
            \Catequil\Db\ConfigProvider::class,
            \Ak\Filter\ConfigProvider::class,
            \Laminas\Filter\ConfigProvider::class,
            \Laminas\Form\ConfigProvider::class,
            \Laminas\InputFilter\ConfigProvider::class,
            ConfigProvider::class,
            new ArrayProvider([
                'development' => true,

                'db' => [
                    'default_connection' => 'memory',

                    'connections' => [
                        'memory' => [
                            'driver' => 'pdo_sqlite',
                            'memory' => true,
                        ],
                        'mysql' => [
                            'driver' => 'pdo_mysql',
                            'user' => 'root',
                            'password' => 'root',
                            'dbname' => 'test',
                            'charset' => 'utf8',
                            'host' => 'localhost',
                        ],
                    ],
                ],
                'orm' => [
                    'connection' => 'db',
                    'proxy_dir' => __DIR__ . '/var/doctrine/proxy',
                    'cache' => [
                        'options' => [
                            'directory' => __DIR__ . '/var/doctrine/metadata',
                        ],
                    ],
                    'entity_paths' => [
                        __NAMESPACE__ => [
                            __DIR__ . '/Entity',
                            __DIR__ . '/Type',
                        ],
                        'AkCustomer\Entity\\' => __DIR__ . '/mock-customer',
                        'AkEmployee\Entity\\' => __DIR__ . '/mock-employee',
                    ],
                ],
                'dependencies' => [
                    'aliases' => [
                        'EventManager' => EventManager::class,
                    ],
                    'invokables' => [
                        EventManager::class,
                    ],
                ],
                'form_elements' => [
                    'aliases' => [
                        ConcreteEntity::class => EntityForm::class,
                        TimestampableEntity::class => EntityForm::class,
                    ],
                ],
            ]),
        ]);
    }

    public function __invoke(): array
    {
        return $this->getMergedConfig();
    }
}
