<?php
namespace AkTest\Orm\Type;

use AkTest\Orm\AbstractTestCase;
use Doctrine\DBAL\Driver\Exception as DBALDriverException;
use Doctrine\DBAL\Exception as DBALException;
use Doctrine\ORM\OptimisticLockException;
use Doctrine\ORM\ORMException;
use Doctrine\ORM\TransactionRequiredException;
use Laminas\Json\Json;

/**
 * Class HydratedObjectTest
 *
 * @package AkTest\Orm\Type
 * @author Danijel Fabijan <hipnaba@gmail.com>
 * @link https://bitbucket.org/autokreso/orm
 */
final class HydratedObjectTest extends AbstractTestCase
{
    /**
     * @throws OptimisticLockException|ORMException|DBALException|DBALDriverException
     */
    public function testCanPersistEntity()
    {
        $entity = new HydratedObjectEntity(new HydratedObjectParams('param name', [
            'first' => 'value 1',
            'second' => 'value 2',
        ]));

        $em = $this->getEntityManager();
        $em->persist($entity);
        $em->flush();

        $conn = $this->getConnection();
        $row = $conn->executeQuery('SELECT * FROM hydrated_object WHERE id = 1')->fetchAssociative();

        $expected = [
            'class' => HydratedObjectParams::class,
            'data' => [
                'name' => 'param name',
                'data' => [
                    'first' => 'value 1',
                    'second' => 'value 2',
                ],
            ],
        ];

        $this->assertEquals($expected, Json::decode($row['params'], Json::TYPE_ARRAY));
    }

    /**
     * @throws OptimisticLockException
     * @throws ORMException
     * @throws TransactionRequiredException
     * @throws DBALException
     */
    public function testCanReadEntity()
    {
        $conn = $this->getConnection();
        $conn->insert('hydrated_object', [
            'id' => 1,
            'params' => Json::encode([
                'class' => HydratedObjectParams::class,
                'data' => [
                    'name' => 'param name',
                    'data' => [
                        'first' => 'value 1',
                        'second' => 'value 2',
                    ],
                ],
            ]),
        ]);

        $em = $this->getEntityManager();
        /** @var HydratedObjectEntity $entity */
        $entity = $em->find(HydratedObjectEntity::class, 1);
        $params = $entity->getParams();

        $this->assertEquals('param name', $params->getName());
        $this->assertEquals([
            'first' => 'value 1',
            'second' => 'value 2',
        ], $params->getData());
    }
}
